@@ -1,47 +1,32 @@
-/*******************************************************************************
- * Licensed Materials - Property of IBM
- * (c) Copyright IBM Corporation 2019. All Rights Reserved.
- *
- * Note to U.S. Government Users Restricted Rights:
- * Use, duplication or disclosure restricted by GSA ADP Schedule
- * Contract with IBM Corp.
- *******************************************************************************/
 /* Copyright (c) 2020 Red Hat, Inc. */
+/* Copyright Contributors to the Open Cluster Management project */
+
 'use strict'
 
 import React from 'react'
-import ReactDOM from 'react-dom'
 import SplitPane from 'react-split-pane'
 import classNames from 'classnames'
 import PropTypes from 'prop-types'
-import {
-  Button,
-  Notification,
-  TextInput,
-  Checkbox,
-  DropdownV2,
-  MultiSelect,
-  ToggleSmall,
-  Modal} from 'carbon-components-react'
-import { Spinner, Tooltip } from '@patternfly/react-core'
-import { initializeControlData, cacheUserData, updateControls, parseYAML } from './utils/update-controls'
+import { Alert, AlertActionCloseButton, Checkbox, ButtonVariant, Spinner, Select,
+  SelectVariant, SelectOption, TextInput, ValidatedOptions, Radio } from '@patternfly/react-core'
+import { AcmModal, AcmButton, AcmAlert } from '@open-cluster-management/ui-components'
+import { initializeControlData, cacheUserData, updateControls, parseYAML,
+  dumpYAMLFromPolicyDiscovered, dumpYAMLFromTemplateObject } from './utils/update-controls'
+import { GrayOutlinedQuestionCircleIcon } from '../Icons'
 import { generateYAML, highlightChanges } from './utils/update-editor'
 import { validateYAML } from './utils/validate-yaml'
-import EditorHeader from './components/EditorHeader'
-import EditorBar from './components/EditorBar'
 import YamlEditor from './components/YamlEditor'
 import './scss/template-editor.scss'
-import msgs from '../../../../nls/platform.properties'
-import '../../../../graphics/diagramIcons.svg'
+import msgs from '../../../nls/platform.properties'
+import '../../../graphics/diagramIcons.svg'
 import { LocaleContext } from '../../../components/common/LocaleContext'
 import _ from 'lodash'
 
 const tempCookie = 'template-editor-open-cookie'
-const diagramIconsInfoStr = '#diagramIcons_info'
+// const diagramIconsInfoStr = '#diagramIcons_info'
 // Regex to test valid policy name format
-// (253 allowable characters, but we have a 'placement-' prefix for PlacementRule,
-// so we're limiting the name input to 243 characters)
-const policyNameRegex = RegExp(/^[a-z0-9][a-z0-9-.]{0,241}[a-z0-9]$/)
+// (it's 63 characters for `${policyNS}.${policyName}`)
+const policyNameRegex = RegExp(/^[a-z0-9][a-z0-9-.]{0,61}[a-z0-9]$/)
 export default class TemplateEditor extends React.Component {
 
   static propTypes = {
@@ -66,8 +51,10 @@ export default class TemplateEditor extends React.Component {
       isFailed: PropTypes.bool,
       error: PropTypes.object
     }),
+    isEdit: PropTypes.bool,
     locale: PropTypes.string,
-    portals: PropTypes.object.isRequired,
+    onCreate: PropTypes.func.isRequired,
+    policyDiscovered: PropTypes.object,
     template: PropTypes.func.isRequired,
     type: PropTypes.string.isRequired,
   }
@@ -75,24 +62,33 @@ export default class TemplateEditor extends React.Component {
   static contextType = LocaleContext
 
   static getDerivedStateFromProps(props, state) {
-    const {fetchControl, createControl={}, createAndUpdateControl={}} = props
+    const {fetchControl, createControl={}, createAndUpdateControl={}, policyDiscovered, locale} = props
     const {isLoaded} = fetchControl || {isLoaded:true}
     const {creationStatus, creationMsg} = createControl
     const {createAndUpdateMsg} = createAndUpdateControl
     if (creationStatus === 'ERROR') {
-      return {updateMsgKind: 'error', updateMessage: creationMsg}
+      return {updateMsgKind: 'danger', updateMessage: creationMsg}
     } else if (createAndUpdateMsg) {
-      return {updateMsgKind: 'error', updateMessage: createAndUpdateMsg}
+      return {updateMsgKind: 'danger', updateMessage: createAndUpdateMsg}
     } else if (isLoaded) {
       const { template, controlData: initialControlData } = props
       let { controlData, templateYAML, templateObject } = state
-
       // initialize controlData, templateYAML, templateObject
       if (!controlData) {
         controlData = initializeControlData(template, initialControlData)
         templateYAML = generateYAML(template, controlData)
         templateObject = parseYAML(templateYAML).parsed
-        return { controlData, templateYAML, templateObject }
+        let isCustomName = false
+        if (policyDiscovered) {
+          templateYAML = dumpYAMLFromPolicyDiscovered(policyDiscovered)
+          templateObject = parseYAML(templateYAML).parsed
+          isCustomName = true
+          if (Object.keys(templateObject).some(item => templateObject[item].length>0)) {
+            controlData = _.cloneDeep(controlData)
+            updateControls(controlData, [], templateObject, locale)
+          }
+        }
+        return { controlData, templateYAML, templateObject, isCustomName }
       }
     }
     return null
@@ -107,11 +103,9 @@ export default class TemplateEditor extends React.Component {
       validPolicyName: true,
       duplicateName: false,
       showEditor,
-      exceptions: [],
       updateMessage: '',
-      hasUndo: false,
-      hasRedo: false,
       resetInx: 0,
+      controlState: {},
     }
     this.multiSelectCmpMap = {}
     this.parseDebounced = _.debounce(()=>{
@@ -125,7 +119,22 @@ export default class TemplateEditor extends React.Component {
   }
 
   componentDidMount() {
-    window.addEventListener('resize',  this.layoutEditors.bind(this))
+    this.props.onCreate(this.handleCreateResource.bind(this))
+  }
+
+  componentDidUpdate(prevProps) {
+    const { isLoaded, isFailed } = this.props.fetchControl || { isLoaded:true }
+    if (!prevProps.fetchControl?.isLoaded && isLoaded && !isFailed) {
+      // if the content was just now loaded, and did not fail.
+      if (window.ResizeObserver) {
+        new ResizeObserver(this.layoutEditors.bind(this))
+          .observe(document.querySelector('.creation-view'))
+      } else {
+        // The ResizeObserver API might not be available on all browsers, but this
+        // is a good fallback covering many cases where the editor should resize
+        window.addEventListener('resize', this.layoutEditors.bind(this))
+      }
+    }
   }
 
   setSplitPaneRef = splitPane => (this.splitPane = splitPane);
@@ -174,10 +183,10 @@ export default class TemplateEditor extends React.Component {
 
     if (isFailed) {
       if (error.name === 'PermissionError') {
-        return <Notification title='' className='overview-notification' kind='error'
+        return <AcmAlert isInline={true} variant='danger'
           subtitle={msgs.get('error.permission.denied.create', locale)} />
       }
-      return <Notification title='' className='overview-notification' kind='error'
+      return <AcmAlert isInline={true} variant='danger'
         subtitle={msgs.get('overview.error.default', locale)} />
     }
 
@@ -188,9 +197,6 @@ export default class TemplateEditor extends React.Component {
     return (
       <div key={`key${resetInx}`} className={viewClasses} ref={this.setContainerRef}>
         {this.conditionalRenderUpdate(locale)}
-        {this.renderEditButton()}
-        {this.renderCreateButton()}
-        {this.renderCancelButton()}
         {this.renderSplitEditor()}
       </div>
     )
@@ -209,6 +215,7 @@ export default class TemplateEditor extends React.Component {
             split="vertical"
             minSize={300}
             maxSize={-500}
+            style={{position:'unset'}}
             ref={this.setSplitPaneRef}
             defaultSize={this.handleSplitterDefault()}
             onChange={this.handleSplitterChange}
@@ -263,46 +270,29 @@ export default class TemplateEditor extends React.Component {
   renderNotifications() {
     const { locale } = this.props
     let { updateMessage, updateMsgKind } = this.state
-    const { validPolicyName, duplicateName } = this.state
+    const { validPolicyName, duplicateName, notificationOpen } = this.state
     // If the name is duplicate and there are no other errors, display an alert
     if ((!updateMessage || updateMsgKind === 'success') && duplicateName) {
       updateMsgKind = 'warning'
       updateMessage = msgs.get('warning.policy.duplicateName', locale)
     }
-    if (updateMessage) {
-      const handleClick = () => {
-        //This is intentional
-      }
-      const handleKeyPress = (e) => {
-        if ( e.key === 'Enter') {
-          handleClick()
-        }
-      }
-
-      return <div role='button' onClick={handleClick}
-        tabIndex="0" aria-label={updateMessage} onKeyDown={handleKeyPress}>
-        <div>
-          <Notification
-            key={updateMessage}
-            kind={updateMsgKind}
-            title={msgs.get(`${updateMsgKind}.create.policy`, locale)}
-            iconDescription=''
-            subtitle={validPolicyName
-              ? updateMessage
-              : <span>
-                  <br />{msgs.get('error.policy.nameFormat.hint', locale)}
-                  <br />{msgs.get('error.policy.nameFormat', locale)}
-                  <br />{msgs.get('error.policy.nameFormat.Rule1', locale)}
-                  <br />{msgs.get('error.policy.nameFormat.Rule2', locale)}
-                  <br />{msgs.get('error.policy.nameFormat.Rule3', locale)}
-                  <br />{msgs.get('error.policy.nameFormat.Rule4', locale)}
-                </span>
-            }
-            className='persistent notification'
-            onCloseButtonClick={this.handleUpdateMessageClosed}
-          />
-        </div>
-      </div>
+    if (updateMessage && notificationOpen || duplicateName) {
+      return (
+        <Alert variant={updateMsgKind} isInline={true} title={msgs.get(`${updateMsgKind}.create.policy`, locale)}
+          actionClose={<AlertActionCloseButton onClose={() => this.setState({ notificationOpen: false, duplicateName: false})} />}>
+          {validPolicyName
+            ? updateMessage
+            : <span>
+                <br />{msgs.get('error.policy.nameFormat.hint', locale)}
+                <br />{msgs.get('error.policy.nameFormat', locale)}
+                <br />{msgs.get('error.policy.nameFormat.Rule1', locale)}
+                <br />{msgs.get('error.policy.nameFormat.Rule2', locale)}
+                <br />{msgs.get('error.policy.nameFormat.Rule3', locale)}
+                <br />{msgs.get('error.policy.nameFormat.Rule4', locale)}
+              </span>
+          }
+        </Alert>
+      )
     }
     return null
   }
@@ -312,18 +302,19 @@ export default class TemplateEditor extends React.Component {
     const { existing, existingByNamespace } = controlData.find(control => control.id === 'name')
     let isDuplicateName = false
     let validPolicyNameFormat = true
-    if (policyName) {
+    // If both a policy name and namespace are provided, check for duplication and formatting
+    if (policyName && policyNS) {
       // Check to see whether the policy name exists already
       if (existing && existingByNamespace) {
         isDuplicateName = existing.includes(policyName) && existingByNamespace[policyName] === policyNS
       }
       // Validate name with RegEx
-      validPolicyNameFormat = policyNameRegex.test(policyName)
+      validPolicyNameFormat = policyNameRegex.test(`${policyNS}.${policyName}`)
     }
     // Set state appropriately
     this.setState({
         validPolicyName: validPolicyNameFormat,
-        duplicateName: isDuplicateName
+        duplicateName: this.props.isEdit ? false: isDuplicateName
     })
   }
 
@@ -341,48 +332,68 @@ export default class TemplateEditor extends React.Component {
           <TextInput
             aria-label={id}
             id={id}
-            invalid={!validPolicyName || duplicateName}
-            hideLabel
-            labelText=''
-            spellCheck={false}
             value={value}
-            onChange={this.onChange.bind(this, id)} />
+            onChange={this.onChange.bind(this, id)}
+            isRequired={mustExist}
+            validated={!validPolicyName?ValidatedOptions.error:duplicateName?ValidatedOptions.warning:ValidatedOptions.default}
+            type="text"
+            isDisabled={this.props.isEdit}
+          />
         </div>
       </React.Fragment>
     )
   }
 
   renderCheckbox(control) {
-    const {id, name, description, checked} = control
+    const {id, name, description, checked, available, active} = control
+    if (id === 'remediation') {
+      const radioButtons=[]
+      available.forEach((item) => {
+        radioButtons.push(<Radio
+            isChecked={item===active}
+            name={`${id}-${item}`}
+            onChange={this.handleChange.bind(this, id)}
+            label={msgs.get(`creation.view.policy.${item}`)}
+            aria-label={`${id}-${item}`}
+            id={`${id}-${item}`}
+            key={`${id}-${item}`}
+            value={item} /> )
+        })
+      return (
+        <React.Fragment>
+          <div className='creation-view-controls-radiobutton'>
+            <div className="creation-view-controls-textbox-title">
+              {name}
+              <div className='creation-view-controls-must-exist'>*</div>
+              <GrayOutlinedQuestionCircleIcon tooltip={description} />
+            </div>
+            {radioButtons}
+          </div>
+        </React.Fragment>
+      )
+    }
     return (
       <React.Fragment>
+        <div className="creation-view-controls-textbox-title">
+          {name}
+          <GrayOutlinedQuestionCircleIcon tooltip={description} />
+        </div>
         <div className='creation-view-controls-checkbox'>
-          <Checkbox
-            aria-label={id}
-            id={id}
-            className='checkbox'
-            hideLabel
-            labelText=''
-            checked={checked}
-            onChange={this.onChange.bind(this, id)} />
-          <div>{name}</div>
-          <Tooltip content={description}>
-            <svg className='info-icon'>
-              <use href={diagramIconsInfoStr} ></use>
-            </svg>
-          </Tooltip>
+          <Checkbox aria-label={id} id={id} isChecked={checked} label={msgs.get(`description.title.${id}`)} onChange={this.onChange.bind(this, id)} />
         </div>
       </React.Fragment>
     )
   }
-
+  onToggle = (id, isOpen) => {
+    this.setState((state) => {
+      state.controlState[id] = { isOpen }
+      return state
+    })
+  }
   renderSingleSelect(control) {
-    const {locale} = this.props
-    const {id, name, available, description, isOneSelection, mustExist} = control
-    let { active } = control
-    // for DropdownV2, empty initialSelectedItem means no pre-selected
-    active = (active && typeof active === 'string') ? active : ''
-    const key = `${id}-${active}`
+    const {locale, isEdit} = this.props
+    const {id, name, available, description, isOneSelection, mustExist, active} = control
+    const {controlState} = this.state
     return (
       <React.Fragment>
         <div className='creation-view-controls-singleselect'
@@ -390,29 +401,41 @@ export default class TemplateEditor extends React.Component {
           <div className="creation-view-controls-multiselect-title">
             {name}
             {mustExist ? <div className='creation-view-controls-must-exist'>*</div> : null}
-            <Tooltip content={description}>
-              <svg className='info-icon'>
-                <use href={diagramIconsInfoStr} ></use>
-              </svg>
-            </Tooltip>
+            <GrayOutlinedQuestionCircleIcon tooltip={description} />
           </div>
-          <DropdownV2
-            aria-label={id}
-            key={key}
-            label={msgs.get('policy.create.namespace.tooltip', locale)}
-            items={available}
-            onChange={this.handleChange.bind(this, id)}
-            initialSelectedItem={active} />
+          <Select
+            variant={SelectVariant.typeahead}
+            onToggle={this.onToggle.bind(this, id)}
+            onSelect={(event, selection)=>{
+              controlState[id] = {
+                selected: selection,
+                isOpen: false,
+              }
+              this.setState({controlState})
+              this.handleChange.bind(this, id)(selection, event)
+            }}
+            selections={active}
+            aria-label={msgs.get('policy.create.namespace.tooltip', locale)}
+            toggleAriaLabel={id}
+            placeholderText={msgs.get('policy.create.namespace.tooltip', locale)}
+            isOpen={controlState[id]&&controlState[id].isOpen}
+            isDisabled={isEdit}
+          >
+            {available.map((option) => (
+              <SelectOption isDisabled={false} key={option} value={option} />
+            ))}
+          </Select>
         </div>
       </React.Fragment>
     )
   }
 
   renderMultiSelect(control) {
-    const {id, name, placeholder:ph, description, isOneSelection, mustExist} = control
+    const {id, name, placeholder:ph, description, isOneSelection, mustExist, active=[]} = control
+    const {controlState} = this.state
 
     // see if we need to add user additions to available (from editing the yaml file)
-    const {active=[], userData, userMap, hasCapturedUserSource} = control
+    const {userData, userMap, hasCapturedUserSource} = control
     let {available, availableMap} = control
     if (userData) {
       if (!hasCapturedUserSource) {
@@ -441,7 +464,7 @@ export default class TemplateEditor extends React.Component {
     }
 
     // change key if active changes so that carbon component is re-created with new initial values
-    const key = `${id}-${active.join('-')}`
+    // const key = `${id}-${active.join('-')}`
     return (
       <React.Fragment>
         <div className='creation-view-controls-multiselect'
@@ -449,42 +472,51 @@ export default class TemplateEditor extends React.Component {
           <div className="creation-view-controls-multiselect-title">
             {name}
             {mustExist ? <div className='creation-view-controls-must-exist'>*</div> : null}
-            <Tooltip content={description}>
-              <svg className='info-icon'>
-                <use href={diagramIconsInfoStr} ></use>
-              </svg>
-            </Tooltip>
+            <GrayOutlinedQuestionCircleIcon tooltip={description} />
           </div>
-          <MultiSelect.Filterable
-            aria-label={id}
-            key={key}
-            items={available}
-            initialSelectedItems={active}
-            placeholder={placeholder}
-            itemToString={item=>item}
-            onChange={this.handleChange.bind(this, id)} />
+          <Select
+            variant={SelectVariant.typeaheadMulti}
+            onToggle={this.onToggle.bind(this, id)}
+            onSelect={(event, selection)=>{
+              if (!active.includes(selection)) {
+                active.push(selection)
+              } else {
+                _.remove(active, (item) => item === selection)
+              }
+              controlState[id] = {
+                selected: active,
+                isOpen: false,
+              }
+              this.setState({controlState})
+              this.handleChange.bind(this, id)(active, event)
+            }}
+            onClear={() => {
+              controlState[id] = {
+                selected: [],
+                isOpen: false,
+              }
+              this.setState({controlState})
+              this.handleChange.bind(this, id)([], event)
+            }}
+            selections={active}
+            isOpen={controlState[id]&&controlState[id].isOpen}
+            aria-label={placeholder}
+            toggleAriaLabel={id}
+            placeholderText={placeholder}
+          >
+            {available.map((option) => (
+              <SelectOption isDisabled={false} key={option} value={option} />
+            ))}
+          </Select>
         </div>
       </React.Fragment>
     )
   }
 
   renderEditor() {
-    const { locale } = this.context
-    const { templateYAML, hasUndo, hasRedo, exceptions } = this.state
+    const { templateYAML } = this.state
     return (
       <div className='creation-view-yaml' >
-        <EditorHeader
-          locale={locale}
-        >
-          <EditorBar
-            hasUndo={hasUndo}
-            hasRedo={hasRedo}
-            exceptions={exceptions}
-            gotoEditorLine={this.gotoEditorLine}
-            handleEditorCommand={this.handleEditorCommand}
-            handleSearchChange={this.handleSearchChange}
-          />
-        </EditorHeader>
         <YamlEditor
           width={'100%'}
           height={'100%'}
@@ -496,54 +528,36 @@ export default class TemplateEditor extends React.Component {
     )
   }
 
-  handleChange(field, evt) {
-    const multiSelect = this.multiSelectCmpMap[field]
-
-    // if this multiselect has an isOneSelection option, close on any selection
-    if (multiSelect) {
-      // if menu is still open don't update until its gone
-      // unfortunately MultiSelect.Filterable doesn't have an onClose
-      const menu = multiSelect.getElementsByClassName('bx--list-box__menu')
-      if (menu && menu.length>0) {
-        multiSelect.selectedItems = evt.selectedItems
-        if (!multiSelect.observer) {
-          multiSelect.observer = new MutationObserver(() => {
-            this.onChange(field, {selectedItems: multiSelect.selectedItems})
-            multiSelect.observer.disconnect()
-            delete multiSelect.observer
-          })
-          multiSelect.observer.observe(menu[0].parentNode, {
-            childList: true
-          })
-        }
-        return
-      }
-    }
-    this.onChange(field, evt)
+  handleChange(field, evt, other) {
+    this.onChange(field, evt, other)
   }
 
-  onChange(field, evt) {
+  onChange(field, evt, other) {
     const { locale } = this.props
     let updateName = false
     let { isCustomName } = this.state
-    const { controlData, templateYAML, updateMessage } = this.state
+    const { controlData, templateYAML, updateMessage, templateObject } = this.state
     const control = controlData.find(({id})=>id===field)
     switch (control.type) {
     case 'text':
-      control.active = evt.target.value
+      control.active = evt.trim()
       isCustomName = field==='name'
       break
     case 'singleselect':
-      control.active = evt.selectedItem
+      control.active = evt
       break
     case 'multiselect':
-      control.active = evt.selectedItems
+      control.active = evt
       // if user was able to select something that automatically
       // generates the name, blow away the user name
       updateName = !isCustomName && control.updateNamePrefix
       break
     case 'checkbox':
-      control.active = evt ? control.available[1] : control.available[0]
+      if (control.name === 'Remediation') {
+        control.active = other.currentTarget.value
+        break
+      }
+      control.active = evt
       control.checked = evt
       break
     }
@@ -562,16 +576,22 @@ export default class TemplateEditor extends React.Component {
         nname.active = cname.toLowerCase()
       }
     }
-
     const {template} = this.props
-    const newYAML = generateYAML(template, controlData)
-    const { parsed: templateObject, exceptions }= parseYAML(newYAML)
+    const newTemplateYAML = generateYAML(template, controlData)
+    const { parsed: newTemplateObject, exceptions }= parseYAML(newTemplateYAML)
+    const finalTemplateObject = _.mergeWith(templateObject, newTemplateObject, (objValue, srcValue, key) => {
+      if (key === 'policy-templates' || key === 'matchExpressions') {
+        return srcValue
+      }
+      return undefined
+    })
+    const finalYaml = dumpYAMLFromTemplateObject(finalTemplateObject)
     // Get highlight decorations
-    const highlights = highlightChanges(this.editor, templateYAML, newYAML)
+    const highlights = highlightChanges(this.editor, templateYAML, finalYaml)
     // Validate YAML and attach highlights and exception decorations
-    validateYAML(templateObject, controlData, exceptions, locale)
+    validateYAML(finalTemplateObject, controlData, exceptions, locale)
     this.handleExceptions(exceptions, highlights)
-    this.setState({controlData, isCustomName, templateYAML: newYAML, templateObject, exceptions})
+    this.setState({controlData, isCustomName, templateYAML: finalYaml, templateObject: finalTemplateObject})
     // If updateMessage is not empty, then there might be a notification message that we'll need to update
     if (updateMessage) {
       this.handleExceptionNotification(exceptions, 'success.edit.policy.check', locale)
@@ -588,20 +608,14 @@ export default class TemplateEditor extends React.Component {
   setEditor = (editor) => {
     this.editor = editor
     this.layoutEditors()
-    editor.onDidChangeModelContent(() => {
-      const model = editor.getModel()
-      const hasUndo = model.canUndo()
-      const hasRedo = model.canRedo()
-      this.setState({hasUndo, hasRedo})
-    })
   }
 
   layoutEditors() {
     if (this.containerRef && this.editor) {
       const controlsSize = this.handleSplitterDefault()
       const rect = this.containerRef.getBoundingClientRect()
-      const width = rect.width - controlsSize - 15
-      const height = rect.height - 80
+      const width = rect.width - controlsSize - 75
+      const height = rect.height
       this.editor.layout({width, height})
     }
   }
@@ -686,8 +700,29 @@ export default class TemplateEditor extends React.Component {
   }
 
   handleEditorChange = (templateYAML) => {
-    this.setState({templateYAML})
-    this.parseDebounced()
+    const { isEdit } = this.props
+    const { templateObject } = this.state
+    const { parsed: newParsed, exceptions } = parseYAML(templateYAML)
+    let undo = false
+    if (isEdit && exceptions.length === 0) {
+      // in edit mode, revert any change to policy/pb/plr name and namespace
+      const yamls = ['Policy', 'PlacementRule', 'PlacementBinding']
+      yamls.forEach(e => {
+        if (_.get(newParsed,`${e}[0].$raw.metadata.name`) !== _.get(templateObject,`${e}[0].$raw.metadata.name`) ||
+          _.get(newParsed,`${e}[0].$raw.metadata.namespace`) !== _.get(templateObject,`${e}[0].$raw.metadata.namespace`)) {
+            undo = true
+        }
+        if (!templateObject[e]) {
+          undo = false
+        }
+      })
+    }
+    if (undo) {
+      this.editor.trigger('api', 'undo')
+    } else {
+      this.setState({templateYAML})
+      this.parseDebounced()
+    }
   }
 
   handleExceptions = (exceptions, decorationList=[]) => {
@@ -721,13 +756,13 @@ export default class TemplateEditor extends React.Component {
   }
 
   handleParse = () => {
-    const {locale}= this.props
+    const { locale }= this.props
     let { isCustomName } = this.state
     const { templateYAML, templateObject, updateMessage } = this.state
     let { controlData } = this.state
 
     // Parse, validate, and add exception decorations
-    const { parsed: newParsed, exceptions} = parseYAML(templateYAML)
+    const { parsed: newParsed, exceptions } = parseYAML(templateYAML)
     validateYAML(newParsed, controlData, exceptions, locale)
     this.handleExceptions(exceptions)
     // If updateMessage is not empty, then there might be a notification message that we'll need to update
@@ -736,13 +771,13 @@ export default class TemplateEditor extends React.Component {
     }
 
     // if no exceptions, update controlData based on custom editing
-    if (Object.keys(newParsed).length>0) {
+    if (Object.keys(newParsed).some(item => newParsed[item].length>0)) {
       controlData = _.cloneDeep(controlData)
       if (updateControls(controlData, templateObject, newParsed, locale)) {
         isCustomName = true
       }
     }
-    this.setState({controlData, isCustomName, templateObject: newParsed, exceptions})
+    this.setState({controlData, isCustomName, templateObject: newParsed})
     // Validate policy name
     this.handlePolicyNameValidation(controlData.find(control => control.id === 'name').active, controlData.find(data => data.id === 'namespace').active)
     return templateYAML // for jest test
@@ -752,9 +787,11 @@ export default class TemplateEditor extends React.Component {
 
   handleExceptionNotification = (exceptions, message, locale) => {
     const errorMsg = exceptions.length>0 ? exceptions[0].text : null
+    const { validPolicyName } = this.state
     this.setState({
-      updateMsgKind: errorMsg?'error':'success',
-      updateMessage: errorMsg||msgs.get(message, locale)
+      updateMsgKind: (errorMsg || !validPolicyName)?'danger':'success',
+      updateMessage: errorMsg||msgs.get(message, locale),
+      notificationOpen: true
     })
     // We're heading to 'Create', so we no longer need the duplicate name alert
     if (!errorMsg && message === 'success.create.policy.check') {
@@ -789,60 +826,6 @@ export default class TemplateEditor extends React.Component {
     return null
   }
 
-  renderEditButton() {
-    const { portals={}, locale } = this.props
-    const { editBtn } = portals
-    if (editBtn) {
-      const portal = document.getElementById(editBtn)
-      if (portal) {
-        const { showEditor } = this.state
-        const handleToggle = () => {
-          if (showEditor) {
-            localStorage.setItem(tempCookie, 'false')
-          } else {
-            localStorage.setItem(tempCookie, 'true')
-          }
-          this.setState({showEditor: !showEditor})
-        }
-        this.renderedPortals = true
-        return ReactDOM.createPortal(
-          <div className='edit-template-switch'>
-            <ToggleSmall
-              id='edit-yaml'
-              ariaLabel={showEditor ? msgs.get('edit.yaml.on', locale) : msgs.get('edit.yaml.off', locale)}
-              defaultToggled={showEditor}
-              onChange={()=>{/*This is intentional*/}}
-              onToggle={handleToggle}
-            />
-            <div className='switch-label'>
-              {showEditor ? msgs.get('edit.yaml.on', locale) : msgs.get('edit.yaml.off', locale)}
-            </div>
-          </div>, portal
-        )
-      }
-    }
-    return null
-  }
-
-  renderCreateButton() {
-    const { portals={}, createControl, locale } = this.props
-    const { createBtn } = portals
-    if (createControl && createBtn) {
-      const portal = document.getElementById(createBtn)
-      if (portal) {
-        return ReactDOM.createPortal(
-          <Button id={`${createBtn}-btn`}
-            onClick={this.handleCreateResource.bind(this)}
-            kind={'primary'} >
-            {msgs.get('button.create', locale)}
-          </Button>,
-          portal
-        )
-      }
-    }
-    return null
-  }
-
   renderUpdatePrompt(locale) {
     let msg = ''
     if (this.state.toCreate && this.state.toUpdate) {
@@ -857,48 +840,56 @@ export default class TemplateEditor extends React.Component {
       }
     }
     return (
-      <Modal
-        danger
+      <AcmModal
+        titleIconVariant='warning'
+        variant='medium'
         id='policy-update-modal'
-        open={this.state.canOpenModal}
-        primaryButtonText={msgs.get('update.apply', locale)}
-        secondaryButtonText={msgs.get('update.cancel', locale)}
-        modalLabel={msgs.get('update.existing', locale)}
-        modalHeading={msgs.get('update.existing', locale)}
-        onRequestClose={() => {
+        isOpen={this.state.canOpenModal}
+        showClose={true}
+        onClose={() => {
           this.setState({ canOpenModal: false })
         }}
-        onSecondarySubmit={() => {
-          this.setState({ canOpenModal: false })
-        }}
-        onRequestSubmit={() => {
-          this.handleUpdateResource(this.state.toCreate, this.state.toUpdate)
-          this.setState({ canOpenModal: false })
-        }}
-        role='region'
-        aria-label={'policy-update'}>
+        title={msgs.get('update.existing', locale)}
+        actions={[
+          <AcmButton key="confirm" variant={ButtonVariant.primary} onClick={() => {
+            this.handleUpdateResource(this.state.toCreate, this.state.toUpdate)
+            this.setState({ canOpenModal: false })
+          }}>
+            {msgs.get('update.apply', locale)}
+          </AcmButton>,
+          <AcmButton key="cancel" variant={ButtonVariant.link} onClick={() => {
+            this.setState({ canOpenModal: false })
+          }}>
+            {msgs.get('update.cancel', locale)}
+          </AcmButton>,
+        ]}>
         <p>{`${msgs.get('update.question', locale)} ${msg}`}</p>
-      </Modal>
+      </AcmModal>
     )
   }
 
   async handleCreateResource() {
-    const { buildControl, createControl } = this.props
+    const { buildControl, createControl, isEdit } = this.props
+    const { validPolicyName } = this.state
     const {createResource} = createControl
     const {buildResourceLists} = buildControl
     const resourceJSON = this.getResourceJSON()
-    if (resourceJSON) {
-      const res = await buildResourceLists(resourceJSON)
-      const create = res.create
-      const update = res.update
-      if (update.length === 0) {
-        createResource(create)
+    if (resourceJSON && validPolicyName) {
+      if (isEdit) {
+        this.handleUpdateResource([], resourceJSON)
       } else {
-        this.setState({
-          canOpenModal: true,
-          toCreate: create,
-          toUpdate: update
-        })
+        const res = await buildResourceLists(resourceJSON)
+        const create = res.create
+        const update = res.update
+        if (update.length === 0) {
+          createResource(create)
+        } else {
+          this.setState({
+            canOpenModal: true,
+            toCreate: create,
+            toUpdate: update
+          })
+        }
       }
     }
   }
@@ -909,34 +900,14 @@ export default class TemplateEditor extends React.Component {
     createAndUpdateResource(create, update)
   }
 
-  renderCancelButton() {
-    const { portals={}, createControl, locale } = this.props
-    const { cancelBtn } = portals
-    if (createControl && cancelBtn) {
-      const portal = document.getElementById(cancelBtn)
-      if (portal) {
-        const {cancelCreate} = createControl
-        return ReactDOM.createPortal(
-          <Button id={`${cancelBtn}-btn`}
-            onClick={cancelCreate}
-            kind={'secondary'} >
-            {msgs.get('button.cancel', locale)}
-          </Button>,
-          portal
-        )
-      }
-    }
-    return null
-  }
-
   resetEditor() {
     const {resetInx} = this.state
     const { template, controlData: initialControlData } = this.props
     const controlData = initializeControlData(template, initialControlData)
     const templateYAML = generateYAML(template, controlData)
     const templateObject = parseYAML(templateYAML).parsed
-    this.setState({controlData, templateYAML, templateObject, exceptions:[],
-      hasUndo: false, hasRedo: false, resetInx:resetInx+1, updateMessage:'',
+    this.setState({controlData, templateYAML, templateObject,
+      resetInx:resetInx+1, updateMessage:'',
       duplicateName: false, validPolicyName: true, isCustomName: false
     })
   }
