@@ -1,21 +1,14 @@
-/*******************************************************************************
- * Licensed Materials - Property of IBM
- * (c) Copyright IBM Corporation 2019. All Rights Reserved.
- *
- * Note to U.S. Government Users Restricted Rights:
- * Use, duplication or disclosure restricted by GSA ADP Schedule
- * Contract with IBM Corp.
- *******************************************************************************/
 /* Copyright (c) 2020 Red Hat, Inc. */
-const config = require('./config'),
+/* Copyright Contributors to the Open Cluster Management project */
+
+const config = require('./server/config'),
       path = require('path'),
       webpack = require('webpack'),
       AssetsPlugin = require('assets-webpack-plugin'),
       CompressionPlugin = require('compression-webpack-plugin'),
       CopyPlugin = require('copy-webpack-plugin'),
       MiniCssExtractPlugin = require('mini-css-extract-plugin'),
-      MonacoWebpackPlugin = require('monaco-editor-webpack-plugin'),
-      TerserPlugin = require('terser-webpack-plugin')
+      MonacoWebpackPlugin = require('monaco-editor-webpack-plugin')
 
 const PRODUCTION = process.env.BUILD_ENV ? /production/.test(process.env.BUILD_ENV) : false
 
@@ -28,12 +21,13 @@ const overpassTest = /overpass-.*\.(woff2?|ttf|eot|otf)(\?.*$|$)/
 module.exports = {
   context: __dirname,
   devtool: PRODUCTION ? 'source-map' : 'cheap-module-source-map',
+  ignoreWarnings: [/Failed to parse source map/],
   stats: {
     children: false,
-    warningsFilter: [/Failed to parse source map/],
+    errorDetails: true,
   },
   entry: {
-    'main': ['@babel/polyfill', './src-web/index.js']
+    'main': ['@babel/polyfill', './src-web/index.js'],
   },
 
   externals: Object.assign(PRODUCTION ? prodExternals : {}, {
@@ -61,11 +55,19 @@ module.exports = {
         // Transpile React JSX to ES5
         test: [/\.jsx$/, /\.js$/],
         exclude: /node_modules|\.scss/,
-        loader: 'babel-loader?cacheDirectory',
-        options: {
-          presets: ['@babel/preset-env', '@babel/preset-react'],
-          plugins: ['@babel/plugin-proposal-class-properties']
-        }
+        use: [
+          {
+            loader: 'babel-loader',
+            options: {
+              cacheDirectory: true,
+              presets: ['@babel/preset-env', '@babel/preset-react'],
+              plugins: [
+                '@babel/plugin-proposal-class-properties',
+                '@babel/plugin-proposal-private-methods',
+              ]
+            },
+          },
+        ]
       },
       {
         test: [/\.s?css$/],
@@ -79,36 +81,32 @@ module.exports = {
             },
           },
           {
-            loader: 'postcss-loader?sourceMap',
+            loader: 'postcss-loader',
             options: {
-              plugins: function () {
-                return [
-                  require('autoprefixer')
-                ]
+              sourceMap: true,
+              postcssOptions: {
+                plugins: [
+                  [
+                    'autoprefixer',
+                  ],
+                ],
               },
             },
           },
           {
-            loader: 'resolve-url-loader',
+            loader: 'sass-loader',
             options: {
               sourceMap: true,
             },
           },
-          {
-            loader: 'sass-loader?sourceMap',
-            options: {
-              prependData: '$font-path: "'+ config.get('contextPath') + '/fonts";'
-            },
-          },
         ],
       },
-      {
-        test: /\.woff2?$/,
-        loader: 'file-loader?name=fonts/[name].[ext]'
-      },
       {
         test: /\.css$/,
-        include: path.resolve(__dirname, './node_modules/monaco-editor'),
+        include: [
+          path.resolve(__dirname, './node_modules/monaco-editor'),
+          path.resolve(__dirname, './node_modules/@open-cluster-management/ui-components'),
+        ],
         use: ['style-loader', 'css-loader'],
       },
       {
@@ -120,13 +118,6 @@ module.exports = {
         test: /\.properties$/,
         loader: 'properties-loader'
       },
-      {
-        test: /\.svg$/,
-        include: path.resolve(__dirname, './graphics'),
-        use: [
-          'svg-sprite-loader'
-        ]
-      },
       {
         test: [/\.handlebars$/, /\.hbs$/],
         loader: 'handlebars-loader',
@@ -136,7 +127,7 @@ module.exports = {
         loader: 'js-yaml-loader',
       },
       {
-        test: /\.(png|jpg|jpeg|gif|svg|woff2?|ttf|eot|otf)(\?.*$|$)/,
+        test: /\.(png|jpg|jpeg|gif|svg|woff|woff2?|ttf|eot|otf)(\?.*$|$)/,
         exclude: [overpassTest, path.resolve(__dirname, './graphics')],
         loader: 'file-loader',
         options: {
@@ -157,20 +148,12 @@ module.exports = {
     ]
   },
 
-  optimization: {
-    minimize: PRODUCTION,
-    minimizer: [new TerserPlugin({
-      parallel: true,
-    })],
-  },
-
   output: {
     //needs to be hash for production (vs chunckhash) in order to cache bust references to chunks
     filename: PRODUCTION ? 'js/[name].[contenthash].min.js' : 'js/[name].js',
     // chunkFilename: PRODUCTION ? 'js/[name].[chunkhash].min.js' : 'js/[name].js',
     path: __dirname + '/public',
     publicPath: config.get('contextPath').replace(/\/?$/, '/'),
-    jsonpFunction: 'webpackJsonpFunctionGrc',
   },
 
   plugins: [
@@ -187,7 +170,6 @@ module.exports = {
     }),
     new MiniCssExtractPlugin({
       filename: PRODUCTION ? 'css/[name].[contenthash].css' : 'css/[name].css',
-      allChunks: true
     }),
     new webpack.LoaderOptionsPlugin({
       options: {
@@ -203,7 +185,7 @@ module.exports = {
       }
     }),
     new CompressionPlugin({
-      filename: '[path].gz',
+      filename: '[path][base].gz',
       algorithm: 'gzip',
       test: /\.js$|\.css$/,
       minRatio: 1,
@@ -219,9 +201,7 @@ module.exports = {
     }),
     new CopyPlugin({
       patterns: [
-        { from: 'node_modules/carbon-icons/dist/carbon-icons.svg', to: 'graphics' },
-        { from: 'graphics', to: 'graphics' },
-        { from: 'fonts', to: 'fonts' },
+        { from: 'src-web/graphics', to: 'graphics' },
       ],
       options: {
         concurrency: 100,
